@@ -9,6 +9,7 @@
 // Copyright 2007      Inge Wallin  <ingwa@kde.org>
 // Copyright 2008, 2009, 2010 Jens-Michael Hoffmann <jmho@c-xx.com>
 // Copyright 2008-2009      Patrick Spendrin <ps_ml@gmx.de>
+// Copyright 2010-2011 Bernhard Beschow  <bbeschow@cs.tu-berlin.de>
 //
 
 #include "MarbleModel.h"
@@ -36,7 +37,6 @@
 #include "GeoSceneLayer.h"
 #include "GeoSceneMap.h"
 #include "GeoScenePalette.h"
-#include "GeoSceneSettings.h"
 #include "GeoSceneTexture.h"
 #include "GeoSceneVector.h"
 #include "GeoSceneXmlDataSource.h"
@@ -62,10 +62,9 @@
 #include "TileCreator.h"
 #include "TileCreatorDialog.h"
 #include "TileLoader.h"
-#include "ViewParams.h"
-#include "ViewportParams.h"
 #include "routing/RoutingManager.h"
 #include "BookmarkManager.h"
+#include "ElevationModel.h"
 
 namespace Marble
 {
@@ -73,29 +72,30 @@ namespace Marble
 class MarbleModelPrivate
 {
  public:
-    MarbleModelPrivate( MarbleModel *parent )
-        : m_parent( parent ),
-          m_clock( new MarbleClock() ),
+    MarbleModelPrivate()
+        : m_clock(),
           m_planet( new Planet( "earth" ) ),
-          m_sunLocator( new SunLocator( m_clock, m_planet ) ),
-          m_pluginManager( new PluginManager( parent ) ),
-          m_mapThemeManager( new MapThemeManager( parent )),
+          m_sunLocator( &m_clock, m_planet ),
+          m_pluginManager(),
+          m_mapThemeManager(),
           m_homePoint( -9.4, 54.8, 0.0, GeoDataCoordinates::Degree ),  // Some point that tackat defined. :-)
           m_homeZoom( 1050 ),
           m_mapTheme( 0 ),
-          m_downloadManager( new HttpDownloadManager( new FileStoragePolicy(
-                                                                   MarbleDirs::localPath() ),
-                                                      m_pluginManager ) ),
+          m_storagePolicy( MarbleDirs::localPath() ),
+          m_downloadManager( &m_storagePolicy, &m_pluginManager ),
+          m_storageWatcher( MarbleDirs::localPath() ),
           m_fileManager( 0 ),
           m_fileviewmodel(),
           m_treemodel(),
           m_descendantproxy(),
           m_sortproxy(),
           m_placemarkselectionmodel( 0 ),
-          m_positionTracking( 0 ),
-          m_bookmarkManager( 0 ),
+          m_positionTracking( &m_treemodel ),
+          m_trackedPlacemark( 0 ),
+          m_bookmarkManager(),
           m_routingManager( 0 ),
-          m_legend( 0 )
+          m_legend( 0 ),
+          m_workOffline( false )
     {
         m_sortproxy.setFilterFixedString( GeoDataTypes::GeoDataPlacemarkType );
         m_sortproxy.setFilterKeyColumn( 1 );
@@ -105,20 +105,15 @@ class MarbleModelPrivate
 
     ~MarbleModelPrivate()
     {
-        delete m_downloadManager;
     }
 
-    void notifyModelChanged();
-
-    MarbleModel             *m_parent;
-
     // Misc stuff.
-    MarbleClock       *const m_clock;
+    MarbleClock              m_clock;
     Planet                  *m_planet;
-    SunLocator        *const m_sunLocator;
+    SunLocator               m_sunLocator;
 
-    PluginManager           *m_pluginManager;
-    MapThemeManager         *m_mapThemeManager;
+    PluginManager            m_pluginManager;
+    MapThemeManager          m_mapThemeManager;
 
     // The home position
     GeoDataCoordinates       m_homePoint;
@@ -127,10 +122,11 @@ class MarbleModelPrivate
     // View and paint stuff
     GeoSceneDocument        *m_mapTheme;
 
-    HttpDownloadManager     *m_downloadManager;
+    FileStoragePolicy        m_storagePolicy;
+    HttpDownloadManager      m_downloadManager;
 
     // Cache related
-    FileStorageWatcher      *m_storageWatcher;
+    FileStorageWatcher       m_storageWatcher;
 
     // Places on the map
     FileManager             *m_fileManager;
@@ -144,42 +140,32 @@ class MarbleModelPrivate
     QItemSelectionModel      m_placemarkselectionmodel;
 
     //Gps Stuff
-    PositionTracking        *m_positionTracking;
+    PositionTracking         m_positionTracking;
 
-    BookmarkManager         *m_bookmarkManager; 
+    const GeoDataPlacemark  *m_trackedPlacemark;
+
+    BookmarkManager          m_bookmarkManager;
     RoutingManager          *m_routingManager;
     QTextDocument           *m_legend;
+
+    bool                     m_workOffline;
+
+    ElevationModel           *m_elevationModel;
 };
 
 MarbleModel::MarbleModel( QObject *parent )
     : QObject( parent ),
-      d( new MarbleModelPrivate( this ) )
+      d( new MarbleModelPrivate() )
 {
-    QTime t;
-    t.start();
-
-    connect(&d->m_treemodel, SIGNAL( dataChanged(QModelIndex,QModelIndex) ),
-            this, SIGNAL( modelChanged() ) );
-    connect(&d->m_treemodel, SIGNAL( layoutChanged() ),
-            this, SIGNAL( modelChanged() ) );
-    connect(&d->m_treemodel, SIGNAL( modelReset() ),
-            this, SIGNAL( modelChanged() ) );
-    connect(&d->m_treemodel, SIGNAL( treeChanged() ),
-            this, SIGNAL( modelChanged() ) );
-
-    // A new instance of FileStorageWatcher.
     // The thread will be started at setting persistent tile cache size.
-    d->m_storageWatcher = new FileStorageWatcher( MarbleDirs::localPath(), this );
     connect( this, SIGNAL( themeChanged( QString ) ),
-             d->m_storageWatcher, SLOT( updateTheme( QString ) ) );
-    // Setting the theme to the current theme.
-    d->m_storageWatcher->updateTheme( mapThemeId() );
+             &d->m_storageWatcher, SLOT( updateTheme( QString ) ) );
+
     // connect the StoragePolicy used by the download manager to the FileStorageWatcher
-    StoragePolicy * const storagePolicy = d->m_downloadManager->storagePolicy();
-    connect( storagePolicy, SIGNAL( cleared() ),
-             d->m_storageWatcher, SLOT( resetCurrentSize() ) );
-    connect( storagePolicy, SIGNAL( sizeChanged( qint64 ) ),
-             d->m_storageWatcher, SLOT( addToCurrentSize( qint64 ) ) );
+    connect( &d->m_storagePolicy, SIGNAL( cleared() ),
+             &d->m_storageWatcher, SLOT( resetCurrentSize() ) );
+    connect( &d->m_storagePolicy, SIGNAL( sizeChanged( qint64 ) ),
+             &d->m_storageWatcher, SLOT( addToCurrentSize( qint64 ) ) );
 
     d->m_fileManager = new FileManager( this );
     d->m_fileviewmodel.setFileManager( d->m_fileManager );
@@ -188,36 +174,28 @@ MarbleModel::MarbleModel( QObject *parent )
     connect( d->m_fileManager,    SIGNAL( fileRemoved(int)),
              &d->m_fileviewmodel, SLOT(remove(int)) );
 
-    d->m_positionTracking = new PositionTracking( this );
+    d->m_routingManager = new RoutingManager( this, this );
 
-    d->m_routingManager = new RoutingManager( d->m_parent, this );
+    connect(&d->m_clock,   SIGNAL( timeChanged() ),
+            &d->m_sunLocator, SLOT( update() ) );
+
+    d->m_elevationModel = new ElevationModel( this );
 
-    connect(d->m_clock,   SIGNAL( timeChanged() ),
-            d->m_sunLocator, SLOT( update() ) );
-     //Initializing Bookmark manager
-    d->m_bookmarkManager = new BookmarkManager();
 }
 
 MarbleModel::~MarbleModel()
 {
-//    mDebug() << "MarbleModel::~MarbleModel";
-
-    delete d->m_positionTracking;
-    delete d->m_bookmarkManager;
-
     delete d->m_fileManager;
     delete d->m_mapTheme;
-    delete d->m_sunLocator;
     delete d->m_planet;
-    delete d->m_clock;
     delete d;
 
     mDebug() << "Model deleted:" << this;
 }
 
-BookmarkManager * MarbleModel::bookmarkManager() const
+BookmarkManager *MarbleModel::bookmarkManager()
 {
-    return d->m_bookmarkManager;
+    return &d->m_bookmarkManager;
 }
 
 QString MarbleModel::mapThemeId() const
@@ -230,7 +208,12 @@ QString MarbleModel::mapThemeId() const
     return mapThemeId;
 }
 
-GeoSceneDocument* MarbleModel::mapTheme() const
+GeoSceneDocument *MarbleModel::mapTheme()
+{
+    return d->m_mapTheme;
+}
+
+const GeoSceneDocument *MarbleModel::mapTheme() const
 {
     return d->m_mapTheme;
 }
@@ -244,10 +227,54 @@ GeoSceneDocument* MarbleModel::mapTheme() const
 // FIXME: Get rid of 'currentProjection' here.  It's totally misplaced.
 //
 
-void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme )
+void MarbleModel::setMapThemeId( const QString &mapThemeId )
 {
-    GeoSceneDocument *oldTheme = d->m_mapTheme;
+    if ( !mapThemeId.isEmpty() && mapThemeId == this->mapThemeId() )
+        return;
+
+    GeoSceneDocument *mapTheme = d->m_mapThemeManager.loadMapTheme( mapThemeId );
+    if ( !mapTheme ) {
+        // Check whether the previous theme works
+        if ( d->m_mapTheme ){
+            qWarning() << "Selected theme doesn't work, so we stick to the previous one";
+            return;
+        }
+
+        // Fall back to default theme
+        QString defaultTheme = "earth/srtm/srtm.dgml";
+        qWarning() << "Falling back to default theme:" << defaultTheme;
+        mapTheme = d->m_mapThemeManager.loadMapTheme( defaultTheme );
+    }
+
+    // If this last resort doesn't work either shed a tear and exit
+    if ( !mapTheme ) {
+        qWarning() << "Couldn't find a valid DGML map.";
+        return;
+    }
+
+    // find the list of previous theme's geodata
+    QStringList loadedContainers;
+    if ( d->m_mapTheme ) {
+        foreach ( GeoSceneLayer *layer, d->m_mapTheme->map()->layers() ) {
+            if ( layer->backend() != dgml::dgmlValue_geodata )
+                continue;
+
+            if ( layer->datasets().count() <= 0 )
+                continue;
+
+            // look for documents
+            foreach ( GeoSceneAbstractDataset *dataset, layer->datasets() ) {
+                if( dataset->fileFormat() == "KML" ) {
+                    QString containername = reinterpret_cast<GeoSceneXmlDataSource*>( dataset )->filename();
+                    loadedContainers <<  containername;
+                }
+            }
+        }
+    }
+
+    delete d->m_mapTheme;
     d->m_mapTheme = mapTheme;
+
     addDownloadPolicies( d->m_mapTheme );
 
     // Some output to show how to use this stuff ...
@@ -274,46 +301,18 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme )
         sunLocator()->setPlanet(d->m_planet);
     }
 
-    // find the list of previous theme's geodata
-    QStringList loadedContainers;
-    QVector<GeoSceneLayer*>::const_iterator it;
-    QVector<GeoSceneLayer*>::const_iterator end;
-    if (oldTheme) {
-        it = oldTheme->map()->layers().constBegin();
-        end = oldTheme->map()->layers().constEnd();
-        for (; it != end; ++it) {
-            GeoSceneLayer* layer = *it;
-            if ( layer->backend() == dgml::dgmlValue_geodata && layer->datasets().count() > 0 ) {
-                // look for documents
-                const QVector<GeoSceneAbstractDataset*> & datasets = layer->datasets();
-                QVector<GeoSceneAbstractDataset*>::const_iterator itds = datasets.constBegin();
-                QVector<GeoSceneAbstractDataset*>::const_iterator endds = datasets.constEnd();
-                for (; itds != endds; ++itds) {
-                    GeoSceneAbstractDataset* dataset = *itds;
-                    if( dataset->fileFormat() == "KML" ) {
-                        QString containername = reinterpret_cast<GeoSceneXmlDataSource*>(dataset)->filename();
-                        loadedContainers <<  containername;
-                    }
-                }
-            }
-        }
-    }
     QStringList loadList;
-    const QVector<GeoSceneLayer*> & layers = d->m_mapTheme->map()->layers();
-    it = layers.constBegin();
-    end = layers.constEnd();
-    for (; it != end; ++it) {
-        GeoSceneLayer* layer = *it;
-        if ( layer->backend() == dgml::dgmlValue_geodata && layer->datasets().count() > 0 ) {
-            // look for documents
-            const QVector<GeoSceneAbstractDataset*> & datasets = layer->datasets();
-            QVector<GeoSceneAbstractDataset*>::const_iterator itds = datasets.constBegin();
-            QVector<GeoSceneAbstractDataset*>::const_iterator endds = datasets.constEnd();
-            for (; itds != endds; ++itds) {
-                GeoSceneAbstractDataset* dataset = *itds;
-                if( dataset->fileFormat() == "KML" ) {
-                    QString containername = reinterpret_cast<GeoSceneXmlDataSource*>(dataset)->filename();
-                    loadedContainers.removeOne( containername );
+    foreach ( GeoSceneLayer *layer, d->m_mapTheme->map()->layers() ) {
+        if ( layer->backend() != dgml::dgmlValue_geodata )
+            continue;
+        if ( layer->datasets().count() <= 0 )
+            continue;
+
+        // look for documents
+        foreach ( GeoSceneAbstractDataset *dataset, layer->datasets() ) {
+            if( dataset->fileFormat() == "KML" ) {
+                QString containername = reinterpret_cast<GeoSceneXmlDataSource*>( dataset )->filename();
+                if ( !loadedContainers.removeOne( containername ) ) {
                     loadList << containername;
                 }
             }
@@ -332,7 +331,7 @@ void MarbleModel::setMapTheme( GeoSceneDocument* mapTheme )
     emit themeChanged( mapTheme->head()->mapThemeId() );
 }
 
-void MarbleModel::home( qreal &lon, qreal &lat, int& zoom )
+void MarbleModel::home( qreal &lon, qreal &lat, int& zoom ) const
 {
     d->m_homePoint.geoCoordinates( lon, lat, GeoDataCoordinates::Degree );
     zoom = d->m_homeZoom;
@@ -350,38 +349,43 @@ void MarbleModel::setHome( const GeoDataCoordinates& homePoint, int zoom )
     d->m_homeZoom = zoom;
 }
 
-MapThemeManager* MarbleModel::mapThemeManager() const
+MapThemeManager *MarbleModel::mapThemeManager()
 {
-    return d->m_mapThemeManager;
+    return &d->m_mapThemeManager;
 }
 
-HttpDownloadManager* MarbleModel::downloadManager() const
+HttpDownloadManager *MarbleModel::downloadManager()
 {
-    return d->m_downloadManager;
+    return &d->m_downloadManager;
+}
+
+const HttpDownloadManager *MarbleModel::downloadManager() const
+{
+    return &d->m_downloadManager;
 }
 
 
-GeoDataTreeModel *MarbleModel::treeModel() const
+GeoDataTreeModel *MarbleModel::treeModel()
 {
     return &d->m_treemodel;
 }
 
-QAbstractItemModel *MarbleModel::placemarkModel() const
+QAbstractItemModel *MarbleModel::placemarkModel()
 {
     return &d->m_sortproxy;
 }
 
-QItemSelectionModel *MarbleModel::placemarkSelectionModel() const
+QItemSelectionModel *MarbleModel::placemarkSelectionModel()
 {
     return &d->m_placemarkselectionmodel;
 }
 
 PositionTracking *MarbleModel::positionTracking() const
 {
-    return d->m_positionTracking;
+    return &d->m_positionTracking;
 }
 
-FileViewModel *MarbleModel::fileViewModel() const
+FileViewModel *MarbleModel::fileViewModel()
 {
     return &d->m_fileviewmodel;
 }
@@ -406,9 +410,9 @@ void MarbleModel::removePlacemarkKey( const QString& key )
     removeGeoData( key );
 }
 
-void MarbleModelPrivate::notifyModelChanged()
+FileManager *MarbleModel::fileManager()
 {
-    emit m_parent->modelChanged();
+    return d->m_fileManager;
 }
 
 qreal MarbleModel::planetRadius()   const
@@ -426,26 +430,34 @@ QString MarbleModel::planetId() const
     return d->m_planet->id();
 }
 
-MarbleClock* MarbleModel::clock() const
+MarbleClock *MarbleModel::clock()
 {
-//    mDebug() << "In dateTime, model:" << this;
-//    mDebug() << d << ":" << d->m_clock;
-    return d->m_clock;
+    return &d->m_clock;
 }
 
-SunLocator* MarbleModel::sunLocator() const
+const MarbleClock *MarbleModel::clock() const
 {
-    return d->m_sunLocator;
+    return &d->m_clock;
+}
+
+SunLocator *MarbleModel::sunLocator()
+{
+    return &d->m_sunLocator;
+}
+
+const SunLocator *MarbleModel::sunLocator() const
+{
+    return &d->m_sunLocator;
 }
 
 quint64 MarbleModel::persistentTileCacheLimit() const
 {
-    return d->m_storageWatcher->cacheLimit() / 1024;
+    return d->m_storageWatcher.cacheLimit() / 1024;
 }
 
 void MarbleModel::clearPersistentTileCache()
 {
-    downloadManager()->storagePolicy()->clearCache();
+    d->m_storagePolicy.clearCache();
 
     // Now create base tiles again if needed
     if ( d->m_mapTheme->map()->hasTextureLayers() ) {
@@ -489,26 +501,37 @@ void MarbleModel::clearPersistentTileCache()
 
 void MarbleModel::setPersistentTileCacheLimit(quint64 kiloBytes)
 {
-    d->m_storageWatcher->setCacheLimit( kiloBytes * 1024 );
+    d->m_storageWatcher.setCacheLimit( kiloBytes * 1024 );
 
     if( kiloBytes != 0 )
     {
-        if( !d->m_storageWatcher->isRunning() )
-            d->m_storageWatcher->start( QThread::IdlePriority );
+        if( !d->m_storageWatcher.isRunning() )
+            d->m_storageWatcher.start( QThread::IdlePriority );
     }
     else
     {
-        d->m_storageWatcher->quit();
+        d->m_storageWatcher.quit();
     }
     // TODO: trigger update
 }
 
-PluginManager* MarbleModel::pluginManager() const
+void MarbleModel::setTrackedPlacemark( const GeoDataPlacemark *placemark )
 {
-    return d->m_pluginManager;
+    d->m_trackedPlacemark = placemark;
+    emit trackedPlacemarkChanged( placemark );
 }
 
-Planet* MarbleModel::planet() const
+const GeoDataPlacemark* MarbleModel::trackedPlacemark() const
+{
+    return d->m_trackedPlacemark;
+}
+
+const PluginManager* MarbleModel::pluginManager() const
+{
+    return &d->m_pluginManager;
+}
+
+const Planet *MarbleModel::planet() const
 {
     return d->m_planet;
 }
@@ -535,7 +558,7 @@ void MarbleModel::addDownloadPolicies( GeoSceneDocument *mapTheme )
     QList<DownloadPolicy *>::const_iterator pos = policies.constBegin();
     QList<DownloadPolicy *>::const_iterator const end = policies.constEnd();
     for (; pos != end; ++pos ) {
-        d->m_downloadManager->addDownloadPolicy( **pos );
+        d->m_downloadManager.addDownloadPolicy( **pos );
     }
 }
 
@@ -544,34 +567,39 @@ RoutingManager* MarbleModel::routingManager()
     return d->m_routingManager;
 }
 
+const RoutingManager* MarbleModel::routingManager() const
+{
+    return d->m_routingManager;
+}
+
 void MarbleModel::setClockDateTime( const QDateTime& datetime )
 {
-    d->m_clock->setDateTime( datetime );
+    d->m_clock.setDateTime( datetime );
 }
 
 QDateTime MarbleModel::clockDateTime() const
 {
-    return d->m_clock->dateTime();
+    return d->m_clock.dateTime();
 }
 
 int MarbleModel::clockSpeed() const
 {
-    return d->m_clock->speed();
+    return d->m_clock.speed();
 }
 
 void MarbleModel::setClockSpeed( int speed )
 {
-    d->m_clock->setSpeed( speed );
+    d->m_clock.setSpeed( speed );
 }
 
 void MarbleModel::setClockTimezone( int timeInSec )
 {
-    d->m_clock->setTimezone( timeInSec );
+    d->m_clock.setTimezone( timeInSec );
 }
 
 int MarbleModel::clockTimezone() const
 {
-    return d->m_clock->timezone();
+    return d->m_clock.timezone();
 }
 
 QTextDocument * MarbleModel::legend()
@@ -586,7 +614,7 @@ void MarbleModel::setLegend( QTextDocument * legend )
 
 void MarbleModel::addGeoDataFile( const QString& filename )
 {
-    d->m_fileManager->addFile( filename, UserDocument );
+    d->m_fileManager->addFile( filename, UserDocument, true );
 }
 
 void MarbleModel::addGeoDataString( const QString& data, const QString& key )
@@ -599,6 +627,30 @@ void MarbleModel::removeGeoData( const QString& fileName )
     d->m_fileManager->removeFile( fileName );
 }
 
+bool MarbleModel::workOffline() const
+{
+    return d->m_workOffline;
+}
+
+void MarbleModel::setWorkOffline( bool workOffline )
+{
+    if ( d->m_workOffline != workOffline ) {
+        downloadManager()->setDownloadEnabled( !workOffline );
+        d->m_workOffline = workOffline;
+        emit workOfflineChanged();
+    }
+}
+
+ElevationModel* MarbleModel::elevationModel()
+{
+    return d->m_elevationModel;
+}
+
+const ElevationModel* MarbleModel::elevationModel() const
+{
+    return d->m_elevationModel;
+}
+
 }
 
 #include "MarbleModel.moc"
