@@ -29,6 +29,8 @@
   #include <QtGui/QPinchGesture>
 #endif
 
+#include "kineticmodel.h"
+
 #include "global.h"
 #include "MarbleDebug.h"
 #include "GeoDataCoordinates.h"
@@ -40,7 +42,6 @@
 #include "AbstractDataPluginItem.h"
 #include "MeasureTool.h"
 #include "MarbleWidgetPopupMenu.h"
-#include "AbstractProjection.h"
 #include "Planet.h"
 
 namespace Marble {
@@ -58,6 +59,7 @@ public:
     QTimer *m_mouseWheelTimer;
     Qt::MouseButtons m_disabledMouseButtons;
     qreal m_wheelZoomTargetDistance;
+    bool m_panViaArrowsEnabled;
 };
 
 MarbleWidgetInputHandler::Protected::Protected( MarbleWidget *widget )
@@ -66,7 +68,8 @@ MarbleWidgetInputHandler::Protected::Protected( MarbleWidget *widget )
       m_positionSignalConnected( false ),
       m_mouseWheelTimer( 0 ),
       m_disabledMouseButtons( Qt::NoButton ),
-      m_wheelZoomTargetDistance( 0.0 )
+      m_wheelZoomTargetDistance( 0.0 ),
+      m_panViaArrowsEnabled( true )
 {
 }
 
@@ -118,6 +121,15 @@ bool MarbleWidgetInputHandler::isMouseButtonPopupEnabled( Qt::MouseButton mouseB
     return !( d->m_disabledMouseButtons & mouseButton );
 }
 
+void MarbleWidgetInputHandler::setPanViaArrowsEnabled( bool enabled )
+{
+    d->m_panViaArrowsEnabled = enabled;
+}
+
+bool MarbleWidgetInputHandler::panViaArrowsEnabled() const
+{
+    return d->m_panViaArrowsEnabled;
+}
 
 class MarbleWidgetDefaultInputHandler::Private
 {
@@ -174,6 +186,9 @@ class MarbleWidgetDefaultInputHandler::Private
     int m_dragThreshold;
     QTimer m_lmbTimer;
 
+    // Models to handle the kinetic spinning.
+    KineticModel m_kineticModel;
+
     QPoint m_selectionOrigin;
     QRubberBand m_selectionRubber;
 
@@ -181,8 +196,6 @@ class MarbleWidgetDefaultInputHandler::Private
     QTimer m_toolTipTimer;
     QPoint m_toolTipPosition;
 
-    MarbleWidgetPopupMenu m_popupmenu;
-
     QAction  *m_addMeasurePointAction;
     QAction  *m_removeLastMeasurePointAction;
     QAction  *m_removeMeasurePointsAction;
@@ -192,8 +205,7 @@ MarbleWidgetDefaultInputHandler::Private::Private( MarbleWidget *widget )
     : m_leftPressed( false ),
       m_midPressed( false ),
       m_dragThreshold( MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ? 15 : 3 ),
-      m_selectionRubber( QRubberBand::Rectangle, widget ),
-      m_popupmenu( widget, widget->model() )
+      m_selectionRubber( QRubberBand::Rectangle, widget )
 {
     m_curpmtl.load( MarbleDirs::path("bitmaps/cursor_tl.xpm") );
     m_curpmtc.load( MarbleDirs::path("bitmaps/cursor_tc.xpm") );
@@ -233,20 +245,20 @@ void MarbleWidgetDefaultInputHandler::Private::ZoomAt(MarbleWidget* marbleWidget
     ViewportParams* now = marbleWidget->viewport();
 
     qreal x(0), y(0);
-    if (!now->currentProjection()->screenCoordinates(destLon, destLat, now, x, y)) {
+    if (!now->screenCoordinates(destLon, destLat, x, y)) {
         return;
     }
 
     ViewportParams soon;
     soon.setProjection(now->projection());
-    soon.setPlanetAxis(now->planetAxis());
+    soon.centerOn(now->centerLongitude(), now->centerLatitude());
     soon.setSize(now->size());
 
     qreal newRadius = marbleWidget->radiusFromDistance(newDistance);
     soon.setRadius( newRadius );
 
     qreal mouseLon, mouseLat;
-    if (!soon.currentProjection()->geoCoordinates(int(x), int(y), &soon, mouseLon, mouseLat, GeoDataCoordinates::Radian )) {
+    if (!soon.geoCoordinates(int(x), int(y), mouseLon, mouseLat, GeoDataCoordinates::Radian )) {
         return;
     }
 
@@ -262,7 +274,7 @@ void MarbleWidgetDefaultInputHandler::Private::ZoomAt(MarbleWidget* marbleWidget
     lookAt.setAltitude( 0.0 );
     lookAt.setRange( newDistance * KM2METER );
 
-    marbleWidget->viewport()->setFocusPoint( GeoDataCoordinates( destLon, destLat ) );
+    marbleWidget->setFocusPoint( GeoDataCoordinates( destLon, destLat ) );
     marbleWidget->flyTo( lookAt, Linear );
 }
 
@@ -302,21 +314,29 @@ MarbleWidgetDefaultInputHandler::MarbleWidgetDefaultInputHandler( MarbleWidget *
     d->m_lmbTimer.setSingleShot(true);
     connect( &d->m_lmbTimer, SIGNAL(timeout()), this, SLOT(lmbTimeout()));
 
+    d->m_kineticModel.setUpdateInterval( 35 );
+    connect( &d->m_kineticModel, SIGNAL( positionChanged( qreal, qreal ) ),
+             MarbleWidgetInputHandler::d->m_widget, SLOT( centerOn( qreal, qreal ) ) );
+    connect( &d->m_kineticModel, SIGNAL( finished() ), SLOT( restoreViewContext() ) );
+
     // The interface to the measure tool consists of a RMB popup menu
     // and some signals.
     MeasureTool *measureTool = MarbleWidgetInputHandler::d->m_widget->measureTool();
 
     // Connect the inputHandler and the measure tool to the popup menu
     d->m_addMeasurePointAction = new QAction( tr( "Add &Measure Point" ), this);
-    d->m_popupmenu.addAction( Qt::RightButton, d->m_addMeasurePointAction );
     d->m_removeLastMeasurePointAction = new QAction( tr( "Remove &Last Measure Point" ),
                                                   this);
     d->m_removeLastMeasurePointAction->setEnabled(false);
-    d->m_popupmenu.addAction( Qt::RightButton, d->m_removeLastMeasurePointAction );
     d->m_removeMeasurePointsAction = new QAction( tr( "&Remove Measure Points" ),
                                                 this);
     d->m_removeMeasurePointsAction->setEnabled(false);
-    d->m_popupmenu.addAction( Qt::RightButton, d->m_removeMeasurePointsAction );
+
+    if ( ! MarbleGlobal::getInstance()->profiles() & MarbleGlobal::SmallScreen ) {
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->addAction( Qt::RightButton, d->m_addMeasurePointAction );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->addAction( Qt::RightButton, d->m_removeLastMeasurePointAction );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->addAction( Qt::RightButton, d->m_removeMeasurePointsAction );
+    }
 
     connect( d->m_addMeasurePointAction, SIGNAL( triggered() ),
              SLOT( addMeasurePoint() ) );
@@ -352,7 +372,7 @@ void MarbleWidgetInputHandler::restoreViewContext()
 
     // Redraw the map with the quality set for Still (if necessary).
     d->m_widget->setViewContext( Still );
-    d->m_widget->viewport()->resetFocusPoint();
+    d->m_widget->resetFocusPoint();
     d->m_wheelZoomTargetDistance = 0.0;
 }
 
@@ -364,7 +384,7 @@ void MarbleWidgetInputHandler::installPluginEventFilter( RenderPlugin *renderPlu
 void MarbleWidgetDefaultInputHandler::showLmbMenu( int x, int y )
 {
     if ( isMouseButtonPopupEnabled( Qt::LeftButton ) ) {
-        d->m_popupmenu.showLmbMenu( x, y );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->showLmbMenu( x, y );
         d->m_toolTipTimer.stop();
     }
 }
@@ -373,7 +393,7 @@ void MarbleWidgetDefaultInputHandler::showRmbMenu( int x, int y )
 {
     if ( isMouseButtonPopupEnabled( Qt::RightButton ) ) {
         d->m_addMeasurePointAction->setData( QPoint( x, y ) );
-        d->m_popupmenu.showRmbMenu( x, y );
+        MarbleWidgetInputHandler::d->m_widget->popupMenu()->showRmbMenu( x, y );
     }
 }
 
@@ -419,7 +439,6 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
          || e->type() == QEvent::MouseButtonRelease )
     {
         QMouseEvent *event = static_cast<QMouseEvent*>( e );
-        QRegion activeRegion = MarbleWidgetInputHandler::d->m_widget->activeRegion();
 
         int dirX = 0;
         int dirY = 0;
@@ -428,7 +447,11 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         if ( event->type() == QEvent::MouseMove
              && !( event->buttons() & Qt::LeftButton ) )
         {
-            d->m_leftPressed = false;
+            if ( d->m_leftPressed ) {
+                d->m_leftPressed = false;
+
+                d->m_kineticModel.release();
+            }
         }
         if ( event->type() == QEvent::MouseMove
              && !( event->buttons() & Qt::MidButton ) )
@@ -452,27 +475,28 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
             }
         }
 
+        qreal mouseLon;
+        qreal mouseLat;
+        const bool isMouseAboveMap = MarbleWidgetInputHandler::d->m_widget->geoCoordinates( event->x(), event->y(),
+                                                 mouseLon, mouseLat,
+                                                 GeoDataCoordinates::Radian );
+
         // emit the position string only if the signal got attached
         if ( MarbleWidgetInputHandler::d->m_positionSignalConnected ) {
-            qreal lat;
-            qreal lon;
-            bool isValid = MarbleWidgetInputHandler::d->m_widget->geoCoordinates( event->x(), event->y(),
-                                                     lon, lat,
-                                                     GeoDataCoordinates::Radian );
-
-            if ( !isValid ) {
+            if ( !isMouseAboveMap ) {
                 emit mouseMoveGeoPosition( tr( NOT_AVAILABLE ) );
             }
             else {
-                QString position = GeoDataCoordinates( lon, lat ).toString();
+                QString position = GeoDataCoordinates( mouseLon, mouseLat ).toString();
                 emit mouseMoveGeoPosition( position );
             }
         }
 
+        QPoint mousePosition( event->x(), event->y() );
 
-        if ( activeRegion.contains( event->pos() ) || d->m_selectionRubber.isVisible() ) {
-
-            // Regarding mouse button presses:
+        if ( isMouseAboveMap || d->m_selectionRubber.isVisible()
+             || MarbleWidgetInputHandler::d->m_widget->whichFeatureAt( mousePosition ).size() != 0) {
+            
             if ( e->type() == QEvent::MouseButtonPress
                  && event->button() == Qt::LeftButton ) {
 
@@ -494,8 +518,12 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 d->m_leftPressedLon = MarbleWidgetInputHandler::d->m_widget->centerLongitude() * DEG2RAD;
                 d->m_leftPressedLat = MarbleWidgetInputHandler::d->m_widget->centerLatitude() * DEG2RAD;
 
+
                 d->m_leftPressedDirection = 1;
 
+                d->m_kineticModel.setPosition( RAD2DEG * ( qreal )( d->m_leftPressedLon ), RAD2DEG * ( qreal )( d->m_leftPressedLat ) );
+                d->m_kineticModel.resetSpeed();
+
                 // Choose spin direction by taking into account whether we
                 // drag above or below the visible pole.
                 if ( MarbleWidgetInputHandler::d->m_widget->projection() == Spherical ) {
@@ -525,6 +553,8 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 d->m_startingRadius = MarbleWidgetInputHandler::d->m_widget->radius();
                 d->m_midPressedY = event->y();
 
+                d->m_kineticModel.release();
+
                 d->m_selectionRubber.hide();
                 MarbleWidgetInputHandler::d->m_widget->setViewContext( Animation );
             }
@@ -554,9 +584,8 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 //as requested
                 emit mouseClickScreenPosition( d->m_leftPressedX, d->m_leftPressedY );
 
-                MarbleWidgetInputHandler::d->m_widget->setViewContext( Still );
-
                 d->m_leftPressed = false;
+                d->m_kineticModel.release();
             }
 
             if ( e->type() == QEvent::MouseButtonRelease
@@ -575,9 +604,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                  && d->m_selectionRubber.isVisible() )
             {
                 qDebug("Marble: Leaving selection");
-                QRect r( MarbleWidgetInputHandler::d->m_widget->mapFromGlobal( d->m_selectionRubber.geometry().topLeft() ),
-                         MarbleWidgetInputHandler::d->m_widget->mapFromGlobal( d->m_selectionRubber.geometry().bottomRight() ));
-                MarbleWidgetInputHandler::d->m_widget->setSelection( r );
+                MarbleWidgetInputHandler::d->m_widget->setSelection( d->m_selectionRubber.geometry() );
                 d->m_selectionRubber.hide();
             }
 
@@ -591,10 +618,11 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                      || abs( deltay ) > d->m_dragThreshold ) {
 
                     d->m_lmbTimer.stop();
-                    MarbleWidgetInputHandler::d->m_widget->centerOn( RAD2DEG * ( qreal )( d->m_leftPressedLon )
-                                                                     - 90.0 * d->m_leftPressedDirection * deltax / radius,
-                                                                     RAD2DEG * ( qreal )( d->m_leftPressedLat )
-                                                                     + 90.0 * deltay / radius );
+
+                    const qreal posLon = RAD2DEG * ( qreal )( d->m_leftPressedLon ) - 90.0 * d->m_leftPressedDirection * deltax / radius;
+                    const qreal posLat = RAD2DEG * ( qreal )( d->m_leftPressedLat ) + 90.0 * deltay / radius;
+                    MarbleWidgetInputHandler::d->m_widget->centerOn( posLon, posLat );
+                    d->m_kineticModel.setPosition( posLon, posLat );
                 }
             }
 
@@ -614,6 +642,8 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
         else {
             d->m_leftPressed = false;
 
+            d->m_kineticModel.release();
+
             QRect boundingRect = MarbleWidgetInputHandler::d->m_widget->mapRegion().boundingRect();
 
             if ( boundingRect.width() != 0 ) {
@@ -635,7 +665,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 dirY = -1;
 
             if ( event->button() == Qt::LeftButton
-                 && e->type() == QEvent::MouseButtonPress ) {
+                 && e->type() == QEvent::MouseButtonPress && panViaArrowsEnabled() ) {
 
                 d->m_lmbTimer.stop();                
                 if ( polarity < 0 )
@@ -649,8 +679,6 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
 
         // Adjusting Cursor shape
 
-        QPoint mousePosition( event->x(), event->y() );
-        
         // Find out if there are data items and if one has defined an action
         QList<AbstractDataPluginItem *> dataItems
             = MarbleWidgetInputHandler::d->m_widget->whichItemAt( mousePosition );
@@ -697,7 +725,11 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                 d->m_arrowCur [1][1] = QCursor( Qt::PointingHandCursor );
         }
 
-        MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowCur[dirX+1][dirY+1] );
+        if ( panViaArrowsEnabled() ) {
+            MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowCur[dirX+1][dirY+1] );
+        } else {
+            MarbleWidgetInputHandler::d->m_widget->setCursor( d->m_arrowCur [1][1] );
+        }
 
         // let others, especially float items, still process the event
         // Mouse move events need to be eaten to avoid the default oxygen behavior of
@@ -741,7 +773,7 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                              destLon, destLat, GeoDataCoordinates::Radian );
 
                 if (isValid) {
-                    marbleWidget->viewport()->setFocusPoint(GeoDataCoordinates(destLon, destLat));
+                    marbleWidget->setFocusPoint(GeoDataCoordinates(destLon, destLat));
                 }
 
                 switch ( pinch->state() ) {
@@ -755,12 +787,12 @@ bool MarbleWidgetDefaultInputHandler::eventFilter( QObject* o, QEvent* e )
                     marbleWidget->setRadius( marbleWidget->radius() * scaleFactor );
                     break;
                 case Qt::GestureFinished:
-                    marbleWidget->viewport()->resetFocusPoint();
+                    marbleWidget->resetFocusPoint();
                     marbleWidget->setViewContext( Still );
                     break;
                 case Qt::GestureCanceled:
                     marbleWidget->setRadius( d->m_startingRadius );
-                    marbleWidget->viewport()->resetFocusPoint();
+                    marbleWidget->resetFocusPoint();
                     marbleWidget->setViewContext( Still );
                     break;
                 }
@@ -834,3 +866,4 @@ void MarbleWidgetDefaultInputHandler::setNumberOfMeasurePoints( int newNumber )
 }
 
 #include "MarbleWidgetInputHandler.moc"
+
