@@ -16,7 +16,6 @@
 #ifndef MARBLE_MERGEDLAYERDECORATOR_H
 #define MARBLE_MERGEDLAYERDECORATOR_H
 
-#include <QtCore/QMutex>
 #include <QtCore/QSharedPointer>
 #include <QtCore/QVector>
 #include <QtGui/QImage>
@@ -25,12 +24,9 @@
 #include "global.h"
 
 class QString;
-class QUrl;
 
 namespace Marble
 {
-class GeoSceneDocument;
-class GeoSceneTexture;
 class SunLocator;
 class StackedTile;
 class TextureTile;
@@ -39,38 +35,39 @@ class TileLoader;
 class MergedLayerDecorator
 {
  public:
-    MergedLayerDecorator( TileLoader * const tileLoader, SunLocator* sunLocator );
+    MergedLayerDecorator( TileLoader * const tileLoader, const SunLocator* sunLocator );
     virtual ~MergedLayerDecorator();
 
-    QImage merge( const TileId id, const QVector<QSharedPointer<TextureTile> > &tiles );
+    QImage merge( const TileId id, const QVector<QSharedPointer<TextureTile> > &tiles ) const;
 
     void setThemeId( const QString &themeId );
 
     void setLevelZeroLayout( int levelZeroColumns, int levelZeroRows );
 
+    void setShowSunShading( bool show );
+    bool showSunShading() const;
+
+    void setShowCityLights( bool show );
+    bool showCityLights() const;
+
     void setShowTileId(bool show);
 
  private:
-    QImage loadDataset( const TileId &id );
     static int maxDivisor( int maximum, int fullLength );
 
-    void initCityLights();
-
-    void paintCityLights( QImage *tileImage, const TileId &id );
     void paintSunShading( QImage *tileImage, const TileId &id ) const;
     void paintTileId( QImage *tileImage, const TileId &id ) const;
 
  protected:
     Q_DISABLE_COPY( MergedLayerDecorator )
     TileLoader * const m_tileLoader;
-    SunLocator* m_sunLocator;
+    const SunLocator* m_sunLocator;
     QString m_themeId;
     int m_levelZeroColumns;
     int m_levelZeroRows;
+    bool m_showSunShading;
+    bool m_showCityLights;
     bool m_showTileId;
-    GeoSceneDocument *m_cityLightsTheme;
-    GeoSceneTexture *m_cityLightsTextureLayer;
-    QMutex m_initCityLightsMutex;
 };
 
 }
