@@ -32,8 +32,11 @@ ScanlineTextureMapperContext::ScanlineTextureMapperContext( StackedTileLoader *
       m_normGlobalWidth( m_globalWidth / ( 2 * M_PI ) ),
       m_normGlobalHeight( m_globalHeight /  M_PI ),
       m_tile( 0 ),
+      m_deltaLevel( 0 ),
       m_tilePosX( 65535 ),
       m_tilePosY( 65535 ),
+      m_vTileStartX( 0 ),
+      m_vTileStartY( 0 ),
       m_toTileCoordinatesLon( 0.5 * m_globalWidth  - m_tilePosX ),
       m_toTileCoordinatesLat( 0.5 * m_globalHeight - m_tilePosY ),
       m_prevLat( 0.0 ),
@@ -66,7 +69,8 @@ void ScanlineTextureMapperContext::pixelValueF( const qreal lon, const qreal lat
         nextTile( posX, posY );
     }
     if ( m_tile ) {
-        *scanLine = m_tile->pixelF( posX, posY );
+        *scanLine = m_tile->pixelF( ( (int)posX + m_vTileStartX ) / ( 1 << m_deltaLevel ),
+                                    ( (int)posY + m_vTileStartY ) / ( 1 << m_deltaLevel ) );
     }
     else {
         *scanLine = 0;
@@ -102,7 +106,8 @@ void ScanlineTextureMapperContext::pixelValue( const qreal lon, const qreal lat,
     }
 
     if ( m_tile ) {
-        *scanLine = m_tile->pixel( iPosX, iPosY );
+        *scanLine = m_tile->pixel( ( iPosX + m_vTileStartX ) >> m_deltaLevel,
+                                   ( iPosY + m_vTileStartY ) >> m_deltaLevel );
     }
     else {
         *scanLine = 0;
@@ -176,21 +181,26 @@ void ScanlineTextureMapperContext::pixelValueApproxF( const qreal lon, const qre
                     nextTile( posX, posY );
                     itLon = prevPixelX + m_toTileCoordinatesLon;
                     itLat = prevPixelY + m_toTileCoordinatesLat;
-                    posX = itLon + itStepLon * j;
-                    posY = itLat + itStepLat * j;
+                    posX = qMax<qreal>( 0.0, qMin<qreal>( tileWidth-1.0, itLon + itStepLon * j ) );
+                    posY = qMax<qreal>( 0.0, qMin<qreal>( tileHeight-1.0, itLat + itStepLat * j ) );
                     oldPosX = -1;
                 }
 
-            *scanLine = m_tile->pixel( posX, posY ); 
+            *scanLine = m_tile->pixel( ( (int)posX + m_vTileStartX ) >> m_deltaLevel,
+                                       ( (int)posY + m_vTileStartY ) >> m_deltaLevel ); 
 
             // Just perform bilinear interpolation if there's a color change compared to the 
             // last pixel that was evaluated. This speeds up things greatly for maps like OSM
             if ( *scanLine != oldRgb ) {
                 if ( oldPosX != -1 ) {
-                    *(scanLine - 1) = m_tile->pixelF( oldPosX, oldPosY, *(scanLine - 1) );
+                    *(scanLine - 1) = m_tile->pixelF( ( oldPosX + m_vTileStartX ) / ( 1 << m_deltaLevel ),
+                                                      ( oldPosY + m_vTileStartY ) / ( 1 << m_deltaLevel ),
+                                                      *(scanLine - 1) );
                     oldPosX = -1;
                 }
-                oldRgb = m_tile->pixelF( posX, posY, *scanLine );
+                oldRgb = m_tile->pixelF( ( posX + m_vTileStartX ) / ( 1 << m_deltaLevel ),
+                                         ( posY + m_vTileStartY ) / ( 1 << m_deltaLevel ),
+                                         *scanLine );
                 *scanLine = oldRgb;
             }
             else {
@@ -302,7 +312,8 @@ void ScanlineTextureMapperContext::pixelValueApprox( const qreal lon, const qrea
             for ( int j = 1; j < n; ++j ) {
                 iPosXf += itStepLon;
                 iPosYf += itStepLat;
-                *scanLine = m_tile->pixel( iPosXf >> 7, iPosYf >> 7 );
+                *scanLine = m_tile->pixel( ( ( iPosXf >> 7 ) + m_vTileStartX ) >> m_deltaLevel,
+                                           ( ( iPosYf >> 7 ) + m_vTileStartY ) >> m_deltaLevel );
                 ++scanLine;
             }
         }        
@@ -323,7 +334,8 @@ void ScanlineTextureMapperContext::pixelValueApprox( const qreal lon, const qrea
                     iPosY = ( itLat + itStepLat * j ) >> 7;
                 }
 
-                *scanLine = m_tile->pixel( iPosX, iPosY );
+                *scanLine = m_tile->pixel( ( iPosX + m_vTileStartX ) >> m_deltaLevel,
+                                           ( iPosY + m_vTileStartY ) >> m_deltaLevel );
                 ++scanLine;
             }
         }
@@ -386,19 +398,19 @@ bool ScanlineTextureMapperContext::isOutOfTileRange( const int itLon, const int
 }
 
 
-int ScanlineTextureMapperContext::interpolationStep( ViewParams * const viewParams )
+int ScanlineTextureMapperContext::interpolationStep( const ViewportParams *viewport, MapQuality mapQuality )
 {
-    if ( viewParams->mapQuality() == PrintQuality ) {
+    if ( mapQuality == PrintQuality ) {
         return 1;    // Don't interpolate for print quality.
     }
 
-    if ( ! viewParams->viewport()->globeCoversViewport() ) {
+    if ( ! viewport->globeCoversViewport() ) {
         return 8;
     }
 
     // Find the optimal interpolation interval m_nBest for the 
     // current image canvas width
-    const int width = viewParams->canvasImage()->width();
+    const int width = viewport->width();
 
     int nBest = 2;
     int nEvalMin = width - 1;
@@ -414,6 +426,18 @@ int ScanlineTextureMapperContext::interpolationStep( ViewParams * const viewPara
 }
 
 
+QImage::Format ScanlineTextureMapperContext::optimalCanvasImageFormat( const ViewportParams *viewport )
+{
+    // If the globe covers fully the screen then we can use the faster
+    // RGB32 as there are no translucent areas involved.
+    QImage::Format imageFormat = ( viewport->mapCoversViewport() )
+                                 ? QImage::Format_RGB32
+                                 : QImage::Format_ARGB32_Premultiplied;
+
+    return imageFormat;
+}
+
+
 void ScanlineTextureMapperContext::nextTile( int &posX, int &posY )
 {
     // Move from tile coordinates to global texture coordinates 
@@ -437,8 +461,8 @@ void ScanlineTextureMapperContext::nextTile( int &posX, int &posY )
     int tileCol = lon / m_tileSize.width();
     int tileRow = lat / m_tileSize.height();
 
-    m_tile = m_tileLoader->loadTile( TileId( 0, m_tileLevel, tileCol, tileRow ) );
-    m_tile->setUsed( true );
+    m_deltaLevel = 0;
+    m_tile = m_tileLoader->loadTile( TileId( 0, m_tileLevel - m_deltaLevel, tileCol >> m_deltaLevel, tileRow >> m_deltaLevel ) );
 
     // Update position variables:
     // m_tilePosX/Y stores the position of the tiles in 
@@ -446,10 +470,12 @@ void ScanlineTextureMapperContext::nextTile( int &posX, int &posY )
     // ( origin upper left, measured in pixels )
 
     m_tilePosX = tileCol * m_tileSize.width();
+    m_vTileStartX = ( tileCol - ( ( tileCol >> m_deltaLevel ) << m_deltaLevel ) ) * m_tileSize.width();
     m_toTileCoordinatesLon = (qreal)(0.5 * m_globalWidth - m_tilePosX);
     posX = lon - m_tilePosX;
 
     m_tilePosY = tileRow * m_tileSize.height();
+    m_vTileStartY = ( tileRow - ( ( tileRow >> m_deltaLevel ) << m_deltaLevel ) ) * m_tileSize.height();
     m_toTileCoordinatesLat = (qreal)(0.5 * m_globalHeight - m_tilePosY);
     posY = lat - m_tilePosY;
 }
@@ -477,8 +503,8 @@ void ScanlineTextureMapperContext::nextTile( qreal &posX, qreal &posY )
     int tileCol = lon / m_tileSize.width();
     int tileRow = lat / m_tileSize.height();
 
-    m_tile = m_tileLoader->loadTile( TileId( 0, m_tileLevel, tileCol, tileRow ) );
-    m_tile->setUsed( true );
+    m_deltaLevel = 0;
+    m_tile = m_tileLoader->loadTile( TileId( 0, m_tileLevel - m_deltaLevel, tileCol >> m_deltaLevel, tileRow >> m_deltaLevel ) );
 
     // Update position variables:
     // m_tilePosX/Y stores the position of the tiles in 
@@ -486,10 +512,12 @@ void ScanlineTextureMapperContext::nextTile( qreal &posX, qreal &posY )
     // ( origin upper left, measured in pixels )
 
     m_tilePosX = tileCol * m_tileSize.width();
+    m_vTileStartX = ( tileCol - ( ( tileCol >> m_deltaLevel ) << m_deltaLevel ) ) * m_tileSize.width();
     m_toTileCoordinatesLon = (qreal)(0.5 * m_globalWidth - m_tilePosX);
     posX = lon - m_tilePosX;
 
     m_tilePosY = tileRow * m_tileSize.height();
+    m_vTileStartY = ( tileRow - ( ( tileRow >> m_deltaLevel ) << m_deltaLevel ) ) * m_tileSize.height();
     m_toTileCoordinatesLat = (qreal)(0.5 * m_globalHeight - m_tilePosY);
     posY = lat - m_tilePosY;
 }
